@@ -338,7 +338,9 @@ static void SAVAGEWakeupHandler( int screenNum, pointer wakeupData,
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    SavagePtr psav = SAVPTR(pScrn);
 
-   DRILock(pScreen, 0);
+   psav->pDRIInfo->wrap.WakeupHandler = psav->coreWakeupHandler;
+   (*psav->pDRIInfo->wrap.WakeupHandler) (screenNum, wakeupData, result, pReadmask);
+   psav->pDRIInfo->wrap.WakeupHandler = SAVAGEWakeupHandler;
    psav->LockHeld = 1;
    if (psav->ShadowStatus) {
       /* fetch the global shadow counter */
@@ -350,7 +352,10 @@ static void SAVAGEWakeupHandler( int screenNum, pointer wakeupData,
 #endif
       psav->ShadowCounter = psav->ShadowVirtual[1023] & 0xffff;
    }
-   psav->AccelInfoRec->NeedToSync = TRUE;
+   if (psav->useEXA)
+	exaMarkSync(pScreen);
+   else
+	psav->AccelInfoRec->NeedToSync = TRUE;
    /* FK: this flag doesn't seem to be used. */
 }
 
@@ -376,7 +381,9 @@ static void SAVAGEBlockHandler( int screenNum, pointer blockData,
       psav->ShadowVirtual[1023] = globalShadowCounter;
    }
    psav->LockHeld = 0;
-   DRIUnlock(pScreen);
+   psav->pDRIInfo->wrap.BlockHandler = psav->coreBlockHandler;
+   (*psav->pDRIInfo->wrap.BlockHandler) (screenNum, blockData, pTimeout, pReadmask);
+   psav->pDRIInfo->wrap.BlockHandler = SAVAGEBlockHandler;
 }
 
 void SAVAGESelectBuffer( ScrnInfoPtr pScrn, int which )
@@ -460,8 +467,8 @@ static Bool SAVAGEDRIAgpInit(ScreenPtr pScreen)
    xf86DrvMsg( pScreen->myNum, X_INFO,
 	       "[agp] Mode 0x%08lx [AGP 0x%04x/0x%04x; Card 0x%04x/0x%04x]\n",
 	       mode, vendor, device,
-	       psav->PciInfo->vendor,
-	       psav->PciInfo->chipType );
+	       VENDOR_ID(psav->PciInfo),
+	       DEVICE_ID(psav->PciInfo));
 
    if ( drmAgpEnable( psav->drmFD, mode ) < 0 ) {
       xf86DrvMsg( pScreen->myNum, X_ERROR, "[agp] AGP not enabled\n" );
@@ -617,7 +624,7 @@ static Bool SAVAGEDRIMapInit( ScreenPtr pScreen )
    pSAVAGEDRIServer->registers.size = SAVAGEIOMAPSIZE;
 
    if ( drmAddMap( psav->drmFD,
-		   (drm_handle_t)psav->MmioBase,
+		   (drm_handle_t)psav->MmioRegion.base,
 		   pSAVAGEDRIServer->registers.size,
 		   DRM_REGISTERS,0,
 		   &pSAVAGEDRIServer->registers.handle ) < 0 ) {
@@ -629,7 +636,7 @@ static Bool SAVAGEDRIMapInit( ScreenPtr pScreen )
    pSAVAGEDRIServer->aperture.size = 5 * 0x01000000;
    
    if ( drmAddMap( psav->drmFD,
-		   (drm_handle_t)(psav->ApertureBase),
+		   (drm_handle_t)(psav->ApertureRegion.base),
 		   pSAVAGEDRIServer->aperture.size,
 		   DRM_FRAME_BUFFER,0,
 		   &pSAVAGEDRIServer->aperture.handle ) < 0 ) {
@@ -875,25 +882,26 @@ Bool SAVAGEDRIScreenInit( ScreenPtr pScreen )
       sprintf(pDRIInfo->busIdString,
               "PCI:%d:%d:%d",
               psav->PciInfo->bus,
+#ifdef XSERVER_LIBPCIACCESS
+              psav->PciInfo->dev,
+#else
               psav->PciInfo->device,
+#endif
               psav->PciInfo->func);
    }
    pDRIInfo->ddxDriverMajorVersion = SAVAGE_VERSION_MAJOR;
    pDRIInfo->ddxDriverMinorVersion = SAVAGE_VERSION_MINOR;
    pDRIInfo->ddxDriverPatchVersion = SAVAGE_PATCHLEVEL;
 
-#if DRIINFO_MAJOR_VERSION < 5
-   pDRIInfo->frameBufferPhysicalAddress = psav->FrameBufferBase;
-#else
-   pDRIInfo->frameBufferPhysicalAddress = (pointer) psav->FrameBufferBase;
-#endif
-
+   pDRIInfo->frameBufferPhysicalAddress = (pointer) psav->FbRegion.base;
    pDRIInfo->frameBufferSize = psav->videoRambytes;
    pDRIInfo->frameBufferStride = pScrn->displayWidth*(pScrn->bitsPerPixel/8);
    pDRIInfo->ddxDrawableTableEntry = SAVAGE_MAX_DRAWABLES;
 
    /* override default DRI block and wakeup handler */
+   psav->coreBlockHandler = pDRIInfo->wrap.BlockHandler;
    pDRIInfo->wrap.BlockHandler = SAVAGEBlockHandler;
+   psav->coreWakeupHandler = pDRIInfo->wrap.WakeupHandler;
    pDRIInfo->wrap.WakeupHandler = SAVAGEWakeupHandler;
 
    pDRIInfo->wrap.ValidateTree = NULL;
@@ -1534,7 +1542,10 @@ SAVAGEDRIMoveBuffers(WindowPtr pParent, DDXPointRec ptOldOrg,
     }
 
     BCI_SEND(0xc0020000); /* wait for 2D idle */
-    psav->AccelInfoRec->NeedToSync = TRUE;
+    if (psav->useEXA)
+	exaMarkSync(pScreen);
+    else
+	psav->AccelInfoRec->NeedToSync = TRUE;
 }
 
 static void 
