@@ -33,6 +33,19 @@
 #include <string.h>
 #include <math.h>
 
+#ifdef XSERVER_LIBPCIACCESS
+#include <pciaccess.h>
+#define VENDOR_ID(p)      (p)->vendor_id
+#define DEVICE_ID(p)      (p)->device_id
+#define SUBSYS_ID(p)      (p)->subdevice_id
+#define CHIP_REVISION(p)  (p)->revision
+#else
+#define VENDOR_ID(p)      (p)->vendor
+#define DEVICE_ID(p)      (p)->chipType
+#define SUBSYS_ID(p)      (p)->subsysCard
+#define CHIP_REVISION(p)  (p)->chipRev
+#endif
+
 #define MODE_24 24
 
 #include "compiler.h"
@@ -178,10 +191,10 @@ typedef struct
 #define SAVAGE_TV_ON	4
 
 #define SAVAGE_DRIVER_NAME	"savage"
-#define SAVAGE_DRIVER_VERSION	"2.1.2"
-#define SAVAGE_VERSION_MAJOR	2
-#define SAVAGE_VERSION_MINOR	1
-#define SAVAGE_PATCHLEVEL	2
+#define SAVAGE_DRIVER_VERSION	PACKAGE_VERSION
+#define SAVAGE_VERSION_MAJOR	PACKAGE_VERSION_MAJOR
+#define SAVAGE_VERSION_MINOR	PACKAGE_VERSION_MINOR
+#define SAVAGE_PATCHLEVEL	PACKAGE_VERSION_PATCHLEVEL
 #define SAVAGE_VERSION	((SAVAGE_VERSION_MAJOR << 24) | \
 			 (SAVAGE_VERSION_MINOR << 16) | \
 			 SAVAGE_PATCHLEVEL)
@@ -267,8 +280,14 @@ typedef struct _StatInfo {
 } StatInfoRec,*StatInfoPtr;
 
 struct savage_region {
-    unsigned        bar;
-    unsigned long   offset;
+#ifdef XSERVER_LIBPCIACCESS
+    pciaddr_t       base;
+    pciaddr_t       size;
+#else
+    unsigned long   base;
+    unsigned long   size;
+#endif
+    void          * memory;
 };
 
 typedef struct _Savage {
@@ -292,18 +311,12 @@ typedef struct _Savage {
     int			endfb;
 
     /* These are physical addresses. */
-    unsigned long	FrameBufferBase;
-    unsigned long	MmioBase;
-    unsigned long	ApertureBase;
     unsigned long	ShadowPhysical;
 
     /* These are linear addresses. */
     struct savage_region   MmioRegion;
     struct savage_region   FbRegion;
     struct savage_region   ApertureRegion;
-    unsigned               last_bar;
-
-    unsigned char*         bar_mappings[3];
 
     unsigned char*	MapBase;
     unsigned char*	BciMem;
@@ -363,8 +376,12 @@ typedef struct _Savage {
     int			TVSizeY;
 
     CloseScreenProcPtr	CloseScreen;
+#ifdef XSERVER_LIBPCIACCESS
+    struct pci_device * PciInfo;
+#else
     pciVideoPtr		PciInfo;
     PCITAG		PciTag;
+#endif
     int			Chipset;
     int			ChipId;
     int			ChipRev;
@@ -444,7 +461,6 @@ typedef struct _Savage {
     __GLXvisualConfig*	pVisualConfigs;
     SAVAGEConfigPrivPtr 	pVisualConfigsPriv;
     SAVAGEDRIServerPrivatePtr DRIServerInfo;
-
     ScreenWakeupHandlerProcPtr coreWakeupHandler;
     ScreenBlockHandlerProcPtr  coreBlockHandler;
 
